#!/usr/bin/env python3

"""
adbreak3

generates SCTE-35 cues for sidecar files
"""

import argparse
import os
from threefive import Cue, SpliceInsert


def argue():
    """
    argue parse command line args
    """
    parser = argparse.ArgumentParser(epilog="adbreak3 is a part of x9k3")
    parser.add_argument(
        "-d",
        "--duration",
        type=float,
        default=60.0,
        help="Set duration of ad break. [ default: 60.0 ]",
    )
    parser.add_argument(
        "-e",
        "--event-id",
        type=int,
        default=1,
        help="Set event id for ad break. [ default: 1 ]",
    )
    parser.add_argument(
        "-i",
        "--cue-in-only",
        action="store_const",
        default=False,
        const=True,
        help="Only make a cue-in SCTE-35 cue [ default: False ]",
    )
    parser.add_argument(
        "-o",
        "--cue-out-only",
        action="store_const",
        default=False,
        const=True,
        help="Only make a cue-out SCTE-35 cue [ default: False ]",
    )
    parser.add_argument(
        "-p",
        "--pts",
        type=float,
        default=0.0,
        help="Set start pts for ad break. Not setting pts will generate a Splice Immediate CUE-OUT. [ default: 0.0 ]",
    )
    parser.add_argument(
        "-P",
        "--preroll",
        action="store_const",
        default=False,
        const=True,
        help=" Add SCTE data four seconds before splice point. Used with MPEGTS. [ default: False ]",
    )
    parser.add_argument(
        "-s",
        "--sidecar",
        default="sidecar.txt",
        help="""Sidecar file of SCTE-35 (pts,cue) pairs. [ default: sidecar.txt ] """,
    )
    parser.add_argument(
        "-v",
        "--version",
        action="store_const",
        default=False,
        const=True,
        help="Show version",
    )

    return parser.parse_args()


def mk_splice_insert(event_id, pts=None, duration=None, out=False):
    """
    mk_splice_insert returns a Cue with a Splice Insert.

    The args set the SpliceInsert vars.

    splice_event_id = event_id

    if pts is None (default):
        splice_immediate_flag      True
        time_specified_flag        False

    if pts:
        splice_immediate_flag      False
        time_specified_flag        True
        pts_time                   pts

    If duration is None (default)
        duration_flag              False

    if duration IS set:
        out_of_network_indicator   True
        duration_flag              True
        break_auto_return          True
        break_duration             duration
        pts_time                   pts

    if out is True:
        out_of_network_indicator   True

    if out is False (default):
        out_of_network_indicator   False

    """
    cue = Cue()
    # default is a CUE-IN
    sin = SpliceInsert()
    sin.splice_event_id = event_id
    sin.splice_event_cancel_indicator = False
    sin.out_of_network_indicator = out
    sin.duration_flag = False
    sin.unique_program_id = event_id
    sin.avail_num = 0
    sin.avails_expected = 0
    sin.splice_immediate_flag = True
    sin.time_specified_flag = False
    sin.program_splice_flag = True
    sin.event_id_compliance_flag = True
    # pts = None for Splice Immediate
    if pts:
        sin.splice_immediate_flag = False
        sin.time_specified_flag = True
        sin.pts_time = float(pts)
    else:
        sin.component_count = 0
    # If we have a duration, set duration
    if duration:
        duration = float(duration)
        sin.break_duration = duration
        sin.break_auto_return = True
        sin.duration_flag = True
        sin.out_of_network_indicator = True
    # Add SpliceInsert to the SCTE35 cue
    cue.command = sin
    cue.encode()
    return cue


def show_cue(new_cue):
    """
    show_cue displays cue data
    """
    cue = Cue(new_cue)
    cue.decode()
    head = "\t\tCUE-OUT"
    body = f"   PTS:{cue.command.pts_time}   Id:{cue.command.splice_event_id}"
    tail = f"   Duration: {cue.command.break_duration}"
    if not cue.command.out_of_network_indicator:
        head = f"\t\tCUE-IN "
        tail = "\n"
    print(head + body + tail)


# cue.command.show()


def cue_out(event_id, pts, duration, sidecar, preroll):
    """
    cue_out generates a SCTE-35 cue with out of network set to True
    """
    if args.pts == 0.0:
        cue = mk_splice_insert(event_id, duration=duration, out=True).encode()
    else:
        cue = mk_splice_insert(event_id, pts=pts, duration=duration, out=True).encode()
    write_line(pts - preroll, cue, sidecar)


def cue_in(event_id, pts, sidecar, preroll):
    """
    cue_in generates a SCTE-35 cue with out of network set to False
    """
    cue = mk_splice_insert(event_id, pts=pts, out=False).encode()
    write_line(pts - preroll, cue, sidecar)


def write_line(pts, cue, sidecar):
    """
    write_line writes pts,cue to sidecar file
    """
    show_cue(cue)
    line = f"{pts},{cue}\n"
    sidecar.write(line)


def mk_cues(args):
    """
    mk_cues generates cues for the sidecar
    based on the values in args
    """
    preroll = 0
    if args.preroll:
        preroll = 4
    event_id = args.event_id
    duration = args.duration
    pts = args.pts
    #   if pts == 0.0:
    #      args.cue_out_only = True
    print(f"\nWriting to sidecar file: {args.sidecar}\n")
    with open(args.sidecar, "a") as sidecar:
        if not args.cue_in_only:
            cue_out(event_id, pts, duration, sidecar, preroll)
            pts = pts + duration
            event_id += 1
        if not args.cue_out_only:
            cue_in(event_id, pts, sidecar, preroll)


def cli():
    args = argue()
    mk_cues(args)


if __name__ == "__main__":
    args = argue()
    mk_cues(args)



#!/usr/bin/env python3

"""
adinterval
generates SCTE-35 cues for sidecar files
"""
import argparse
import os, sys, time
from threefive import Cue
from threefive.encode import mk_splice_insert
from threefive import IFramer
from threefive import reader

ROLLOVER = 95443.717678


class Args:
    """
    Args sets Break Duration and Interval
    """

    def __init__(self):
        self.event_id = 1
        self.input = None
        self.duration = None  # Break Duration
        self.pts = None
        self.bval = None  # Break Interval

    def load(self, parsed):
        self.input = parsed.input
        self.duration = float(parsed.duration)
        self.bval = float(parsed.break_interval)
        self.sidecar = parsed.sidecar

    def iter_args(self):
        """
        iter_args iterates pts, event_id,
        handles clock rollover,
        and sleeps during the break interval
        """
        self.pts += self.bval
        self.pts = self.pts % ROLLOVER
        self.event_id += 1
        time.sleep((self.duration + self.bval) - 4)


def argue():
    """
    argue parses command line args
    """
    parser = argparse.ArgumentParser(epilog="adinterval3 is part of x9k3")
    parser.add_argument(
        "-i",
        "--input",
        default=None,
        help=f"Input source, is a master.m3u8(local or http(s) with MPEGTS segments  default: None",
    )
    parser.add_argument(
        "-s",
        "--sidecar",
        default="sidecar.txt",
        help=f"SCTE-35 Sidecar file default: None",
    )
    parser.add_argument(
        "-b",
        "--break_interval",
        default=300,
        help=f"Interval between breaks default: 300",
    )
    parser.add_argument(
        "-d",
        "--duration",
        default=60,
        help=f"Interval between breaks default: 60",
    )
    parsed = parser.parse_args()
    args = Args()
    args.load(parsed)
    args.pts = find_iframe_pts(args.input)
    return args


def show_cue(new_cue):
    """
    show_cue displays cue data
    """
    cue = Cue(new_cue)
    cue.decode()
    cue.show()


def cue_out(event_id, pts, duration, sidecar):
    """
    cue_out generates a SCTE-35 cue with out of network set to True
    """
    cue = mk_splice_insert(event_id, pts=pts, duration=duration, out=True).encode()
    write_line(pts, cue, sidecar, "CUE-OUT")


def cue_in(event_id, pts, sidecar):
    """
    cue_in generates a SCTE-35 cue with out of network set to False
    """
    cue = mk_splice_insert(event_id, pts=pts, duration=None, out=False).encode()
    write_line(pts, cue, sidecar, "CUE-IN")


def write_line(pts, cue, sidecar, cue_state):
    """
    write_line writes pts,cue to sidecar file
    """
    fixed = round(pts, 6)
    print(f"{cue_state} @ {fixed}")
    line = f"{fixed},{cue}\n"
    sidecar.write(line)
    sidecar.flush()


def mk_cues(args):
    """
    mk_cues generates cues for the sidecar
    based on the values in args
    """
    while True:
        with open(args.sidecar, "w") as blank:
            blank.close()
        with open(args.sidecar, "a") as sidecar:
            cue_out(args.event_id, args.pts, args.duration, sidecar)
            args.pts += args.duration
            cue_in(args.event_id, args.pts, sidecar)
            sidecar.close()
            args.iter_args()


def find_iframe_pts(manifest):
    with reader(manifest) as m3u8:
        segs = [line for line in m3u8.readlines() if line and line[0] != "#"]
        ifr = IFramer()
        seg = segs[-1].strip().decode()
        pts = ifr.first(seg) + 30.000
        print("START", pts)
        return pts


def cli():
    args = argue()
    mk_cues(args)


if __name__ == "__main__":
    args = argue()
    mk_cues(args)
